@@ -6,10 +6,16 @@
 #define V8_HEAP_CPPGC_JS_UNIFIED_HEAP_MARKING_VERIFIER_H_
 
 #include "src/heap/cppgc/marking-verifier.h"
+#include "v8-traced-handle.h"
 
 namespace v8 {
 namespace internal {
 
+class UnifiedHeapVerificationState : public cppgc::internal::VerificationState {
+ public:
+  void VerifyMarkedTracedReference(const TracedReferenceBase& ref) const;
+};
+
 class V8_EXPORT_PRIVATE UnifiedHeapMarkingVerifier final
     : public cppgc::internal::MarkingVerifierBase {
  public:
@@ -18,9 +24,7 @@ class V8_EXPORT_PRIVATE UnifiedHeapMarkingVerifier final
   ~UnifiedHeapMarkingVerifier() final = default;
 
  private:
-  // TODO(chromium:1056170): Use a verification state that can handle JS
-  // references.
-  cppgc::internal::VerificationState state_;
+  UnifiedHeapVerificationState state_;
 };
 
 }  // namespace internal
